@@ -29,8 +29,8 @@ struct _VoiceWindow {
 };
 
 struct _LocationInfo {
-	float lat;
-	float lon;
+        gdouble *lat;
+	gdouble *lon;
 	gchar *city;
 };
 
