@@ -3,6 +3,7 @@
 #include <gst/player/player.h>
 #include <champlain/champlain.h>
 #include <math.h>
+#include <time.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <glib/gstdio.h>
@@ -12,6 +13,8 @@
 #include <geoclue.h>
 #include <champlain-gtk/champlain-gtk.h>
 #include <clutter-gtk/clutter-gtk.h>
+#include <libsoup/soup.h>
+#include <libsoup/soup-message.h>
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -34,8 +37,12 @@ GtkWidget *filename_entry;
 GtkWidget *filename_label;
 GtkWidget *summary_entry;
 GtkWidget *summary_label;
+GtkTextBuffer *summary;
 gchar *voice_filename;
 gchar *voice_stream;
+char timebuffer[256];
+GDateTime *t_timebuffer;
+GDate d;
 
 VoiceInfo *voiceinfo;
 
@@ -58,7 +65,7 @@ marker_function (ChamplainMarker *self,
 		 ClutterEvent    *event,
 		 gpointer         user_data)
 {
-        gchar *station, *station_link;
+        gchar *station = NULL, *station_link;
 	gchar *markup;
 	guint context_id;
 	GstPlayer *player;
@@ -66,19 +73,15 @@ marker_function (ChamplainMarker *self,
 	player = gst_player_new (NULL, gst_player_g_main_context_signal_dispatcher_new(NULL));
 	voinfo = gnome_voice_file_loader (voinfo, voice_filename);
 	while (voinfo != NULL) {
-	         if (strcasecmp(voinfo->location, station_link)==0) {
-		        station = (gchar *)champlain_label_get_text (CHAMPLAIN_LABEL (self));
-		        station_link = strtok(station, "\n");
-		        player = gst_player_new (NULL, gst_player_g_main_context_signal_dispatcher_new(NULL));
-			gst_player_set_uri (GST_PLAYER (player), voinfo->stream);
-			gst_player_play(GST_PLAYER (player));
-		}
-		voinfo = voinfo->next;
+	  if (strcasecmp(voinfo->location, station_link)==0) {
+	    station = (gchar *)champlain_label_get_text (CHAMPLAIN_LABEL (self));
+	    station_link = strtok(station, "\n");
+	    player = gst_player_new (NULL, gst_player_g_main_context_signal_dispatcher_new(NULL));
+	    gst_player_set_uri (GST_PLAYER (player), voinfo->stream);
+	    gst_player_play(GST_PLAYER (player));
+	  }
+	  voinfo = voinfo->next;
 	}
-	if (user_data != NULL) {
-		champlain_label_set_text (CHAMPLAIN_LABEL (self), user_data);
-	}
-	/* g_signal_connect(CHAMPLAIN_LOCATION(self), "button-press", G_CALLBACK (on_new_station_clicked), user_data); */
 	return;
 }
 
@@ -87,31 +90,44 @@ create_marker_layer (G_GNUC_UNUSED ChamplainView *view, ChamplainPathLayer **pat
 {
 	ClutterActor *marker;
 	ClutterActor *layer_actor;
-	ClutterColor city_color = { 0x10, 0x10, 0x10, 0xbb };
-	ClutterColor city_a_color = { 0x36, 0x89, 0xe6, 0xbb };
-	ClutterColor city_b_color = { 0xde, 0x3e, 0x80, 0xbb };
-	ClutterColor city_c_color = { 0xa5, 0x6d, 0xe2, 0xbb };
-	ClutterColor city_d_color = { 0x71, 0x53, 0x44, 0xbb };
-	ClutterColor city_e_color = { 0xab, 0xac, 0xae, 0xbb };
-	ClutterColor city_f_color = { 0x48, 0x5a, 0x6c, 0xbb };
+	ChamplainLabel *self;
+	ClutterColor city_color = { 0x10, 0x10, 0xff, 0xbb };
+	ClutterColor voicegram_color = { 0x00, 0xff, 0x10, 0xbb };	
 	ClutterColor text_color = { 0xff, 0xff, 0xff, 0xff };
 	LocationCallbackData callback_data;
-	gchar *station = NULL;
+	gchar *station = NULL, *station_link;
+	VoiceInfo *voinfo = gnome_voice_file_loader (voinfo, voice_filename);
 	*path = champlain_path_layer_new ();
+	champlain_path_layer_set_stroke_color(path, &city_color);
 	layer = champlain_marker_layer_new_full (CHAMPLAIN_SELECTION_SINGLE);
-	layer_actor = CLUTTER_ACTOR (layer);
-	marker = champlain_label_new_from_file ("icons/emblem-generic.png", NULL);
-	station = g_strdup("Los Angeles, California\n<span size=\"small\">Sondre Lerche</span>");
-	champlain_label_set_text (CHAMPLAIN_LABEL (marker), station);
-	champlain_label_set_use_markup (CHAMPLAIN_LABEL (marker), TRUE);
-	champlain_label_set_color (CHAMPLAIN_LABEL (marker), &city_a_color);
-	champlain_label_set_text_color (CHAMPLAIN_LABEL (marker), &text_color);
-	champlain_location_set_location (CHAMPLAIN_LOCATION (marker), lat_gps, lon_gps);
-	champlain_marker_layer_add_marker (layer, CHAMPLAIN_MARKER (marker));
-	champlain_marker_animate_in(CHAMPLAIN_MARKER (marker));
-	/* champlain_path_layer_add_node (*path, CHAMPLAIN_LOCATION (marker)); */
-	g_signal_connect(CHAMPLAIN_LOCATION(marker), "button-press", G_CALLBACK(marker_function), station);
-	champlain_marker_layer_set_all_markers_draggable (layer);
+	layer_actor = CLUTTER_ACTOR (layer);	
+	while (voinfo != NULL) {
+	  marker = champlain_label_new_from_file ("icons/emblem-generic.png", NULL);
+	  station = g_strdup(g_strconcat(voinfo->name, "\n<span size=\"small\">", voinfo->location->city, g_strconcat("</span>\n<span size=\"x-small\">", voinfo->uri, "</span>", NULL), NULL));
+	  champlain_label_set_text (CHAMPLAIN_LABEL (marker), station);
+	  champlain_label_set_use_markup (CHAMPLAIN_LABEL (marker), TRUE);
+	  champlain_label_set_color (CHAMPLAIN_LABEL (marker), &city_color);
+	  champlain_label_set_text_color (CHAMPLAIN_LABEL (marker), &text_color);
+	  champlain_location_set_location (CHAMPLAIN_LOCATION (marker), atof(voinfo->location->lat), atof(voinfo->location->lon));
+	  champlain_marker_layer_add_marker (layer, CHAMPLAIN_MARKER (marker));
+	  champlain_marker_animate_in(CHAMPLAIN_MARKER (marker));
+	  champlain_path_layer_add_node (*path, CHAMPLAIN_LOCATION (marker));
+	  g_signal_connect(CHAMPLAIN_LOCATION(marker), "button-press", G_CALLBACK(marker_function), station);
+	  voinfo = voinfo->next;
+	}
+	voinfo = gnome_voice_file_loader (voinfo, g_strconcat (GNOME_VOICE_DATADIR, "/gnome-voice.xml", NULL));
+	while (voinfo != NULL) {
+	  marker = champlain_label_new_from_file ("icons/emblem-generic.png", NULL);
+	  station = g_strdup(g_strconcat(voinfo->name, "\n<span size=\"small\">", voinfo->location->city, g_strconcat("</span>\n<span size=\"x-small\">", voinfo->uri, "</span>", NULL), NULL));
+	  champlain_label_set_text (CHAMPLAIN_LABEL (marker), station);
+	  champlain_label_set_use_markup (CHAMPLAIN_LABEL (marker), TRUE);
+	  champlain_label_set_color (CHAMPLAIN_LABEL (marker), &voicegram_color);
+	  champlain_label_set_text_color (CHAMPLAIN_LABEL (marker), &text_color);
+	  champlain_location_set_location (CHAMPLAIN_LOCATION (marker), atof(voinfo->location->lat), atof(voinfo->location->lon));
+	  champlain_marker_layer_add_marker (layer, CHAMPLAIN_MARKER (marker));
+	  champlain_marker_animate_in(CHAMPLAIN_MARKER (marker));
+	  voinfo = voinfo->next;
+	}
 	clutter_actor_show (layer_actor);
 	return layer;
 }
@@ -287,7 +303,6 @@ search_text_changed (GtkEntry *entry, VoiceWindow *window)
 	const gchar *text;
 	GtkWidget *tab;
 	GtkWidget *view;
-	GtkTextBuffer *buffer;
 	GtkTextIter start, match_start, match_end;
 	text = gtk_entry_get_text (filename_entry);
 	if (text[0] == '\0')
@@ -476,10 +491,9 @@ gps_callback (GClueSimple *simple, GpsCallbackData *data)
         GTimeVal tv = { 0 };
         const char *desc;
 
-#if 0
 	lat_gps = gclue_location_get_latitude (location);
 	lon_gps = gclue_location_get_longitude (location);
-#endif
+
 	champlain_view_center_on (CHAMPLAIN_VIEW (data->view), lat_gps, lon_gps);
 	champlain_location_set_location (CHAMPLAIN_LOCATION (data->voice_marker), lat_gps, lon_gps);
 
@@ -576,37 +590,47 @@ static void gv_wizard_close(GtkAssistant * assistant, gpointer data)
 {
 	FILE *voice_pointer = NULL;
 	GDateTime *datestamp = g_date_time_new_now_utc ();
-	gchar *filename_voice = g_strconcat(g_get_user_special_dir(G_USER_DIRECTORY_MUSIC), "/", "GNOME.voice", NULL);
+	gchar *filename_voice = g_strconcat(g_get_user_special_dir(G_USER_DIRECTORY_MUSIC), "/", g_get_real_name(), ".voice", NULL);
 	gchar *string;
-	char buf[256];
-	GDate d;
 	const char *charset;
-        voiceinfo = (VoiceInfo *) data;
-
-	/* Version and current date. */
-	g_date_clear (&d, 1);
-	g_date_set_time_t (&d, time (NULL));
-	g_date_strftime (buf, sizeof (buf), "%s", &d);
-	
+	guint status;
+	SoupMessage *msg;
+	SoupMessageHeaders *response_headers;
+	SoupSession *session;
+	GError *error = NULL;
+	GBytes *bytes;
+	gchar *voicegram_save_url;
+	voiceinfo = (VoiceInfo *) data;
+	session = soup_session_sync_new ();
 	voice_pointer = fopen(filename_voice, "w");
 	fprintf(voice_pointer, "<voice version='%s'>\n", VERSION);
 	fprintf(voice_pointer, "  <station name='%s' uri='http://%s/GNOME.ogg'>\n",
 		g_get_real_name(),
 		g_get_host_name());
-	fprintf(voice_pointer, "    <location>%s</location>\n", g_get_host_name());
-	fprintf(voice_pointer, "    <stream>https://www.gnomevoice.org/stream/data/%s%s</stream>\n", buf, ".voice");
+	fprintf(voice_pointer, "    <location lat='%10.7f' lon='%9.7f'>%s</location>\n", lon_gps, g_get_host_name());
+	fprintf(voice_pointer, "    <stream>https://www.gnomevoice.org/stream/data/%s%s</stream>\n", timebuffer, ".voice");
 	fprintf(voice_pointer, "  </station>\n");
 	fprintf(voice_pointer, "</voice>\n");
 	fclose(voice_pointer);
+	voicegram_save_url = g_strconcat("https://www.gnomevoice.org/stream/?notBefore=", g_date_time_format_iso8601 (t_timebuffer), g_strconcat("&notAfter=", g_date_time_format_iso8601 (t_timebuffer), "&label=GNOME&sofset=0&eofset=", strlen(voice_pointer), "&MAX_FILE_SIZE=5000000&userfile=", NULL), filename_voice, NULL);
+	g_print ("Voicegram GET %s\n", voicegram_save_url);
+	msg = soup_message_new ("GET", voicegram_save_url);
+	soup_message_set_request (msg, "application/xml",
+				  SOUP_MEMORY_COPY,
+				  voice_pointer,
+				  strlen (voice_pointer));
+	status = soup_session_send_message (session, msg);
+	g_object_unref (msg);
+	g_object_unref (session);
 	g_date_time_unref (datestamp);
-        if (!main_loops) {
-                g_error("Quit more loops than there are.");
+	if (!main_loops) {
+	         g_error("Quit more loops than there are.");
         } else {
                 GMainLoop *loop = main_loops;
                 g_main_loop_quit(loop);
                 gtk_main_quit();
         }
-	// gst_element_send_event(data, gst_event_new_eos());
+	/* gst_element_send_event(data, gst_event_new_eos()); */
 }
 
 static void gv_wizard_apply(GtkAssistant * assistant, gpointer data)
@@ -659,28 +683,37 @@ main (gint argc, gchar **argv)
 	GtkWidget *introduction;
 	int i = 0;
 	PageInfo page[5] = {
-		{NULL, -1, "Voice 1.0.1", GTK_ASSISTANT_PAGE_INTRO, TRUE},
-		{NULL, -1, "Title", GTK_ASSISTANT_PAGE_CONTENT, TRUE},
+		{NULL, -1, "Voice 1.1.0", GTK_ASSISTANT_PAGE_INTRO, TRUE},
+		{NULL, -1, "Voicegram", GTK_ASSISTANT_PAGE_CONTENT, TRUE},
 		{NULL, -1, "Filename", GTK_ASSISTANT_PAGE_CONTENT, TRUE},
 		{NULL, -1, "Summary", GTK_ASSISTANT_PAGE_SUMMARY, TRUE},		
-		{NULL, -1, "Complete", GTK_ASSISTANT_PAGE_CONFIRM, TRUE},		
+		{NULL, -1, "Finish", GTK_ASSISTANT_PAGE_CONFIRM, TRUE},
 	};
 	VoiceInfo *voinfo;
 	int entries_n = 2;
 	int entryNumber;
 	GError *error = NULL;
 	GOptionContext *context;
-
+	GtkTextBuffer *buffer;
+	char *basename;
+	GtkWidget *scrolled, *textview;
+	char *contents;
+	gsize length;
+	
+	/* Version and current date. */
+	g_date_clear (&d, 1);
+	g_date_set_time_t (&d, time (NULL));
+	g_date_strftime (timebuffer, sizeof (timebuffer), "%s", &d);
+	t_timebuffer = g_date_time_new_now_utc ();
 	voice_filename = g_strconcat(g_get_user_special_dir(G_USER_DIRECTORY_MUSIC), "/",
-					    "GNOME.voice", NULL);
-	voinfo = gnome_voice_file_loader (voinfo,
-					  voice_filename);
+				     g_get_real_name(), ".voice", NULL);
+	voinfo = gnome_voice_file_loader (voinfo, voice_filename);
 	// gtk_init(&argc, &argv);
 	if (argc < 2) {
 	  fprintf(stderr, "No Voice Stream typed on the command line\n");
 	}
 
-	context = g_option_context_new ("- Voice 1.0.1");
+	context = g_option_context_new ("- Voice 1.1.0");
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	if (!g_option_context_parse (context, &argc, &argv, &error)) {
@@ -693,14 +726,15 @@ main (gint argc, gchar **argv)
 	introduction = gtk_assistant_new();
 	gtk_container_add (GTK_WINDOW (window), introduction);
 	gtk_widget_set_size_request(GTK_WIDGET(introduction), 640, 480);
-	gtk_window_set_title(GTK_WINDOW(introduction), "Voice 1.0.1");
+	gtk_window_set_title(GTK_WINDOW(introduction), "Voice 1.1.0");
 	g_signal_connect(G_OBJECT(introduction), "destroy",
 			 G_CALLBACK(gtk_main_quit), NULL);
-	page[0].widget = gtk_label_new(_("Welcome to Voice 1.0.1!\n\nRecord respectfully around others.\n\nClick Next to setup a voice recording session!\n\nClick Cancel to stop the voice recording session.\n\nClick Cancel twice to exit Voice."));
+	page[0].widget = gtk_label_new(_("Welcome to Voice 1.1.0!\n\nRecord respectfully around others.\n\nClick Next to setup a voice recording session!\n\nClick Cancel to stop the voice recording session.\n\nClick Cancel twice to exit Voice."));
 	page[1].widget = gtk_box_new(FALSE, 5);
-	title_label = gtk_label_new(_("Title:"));
+	title_label = gtk_label_new(_("Voicegram:"));
 	title_entry = gtk_entry_new();
-	if (g_strcmp0(title_entry, NULL)!=0) gtk_entry_set_text(GTK_ENTRY(title_entry), g_get_real_name()); else gtk_entry_set_text(GTK_ENTRY(title_entry), gtk_entry_get_text(GTK_ENTRY(title_entry)));
+	gtk_entry_set_max_length (title_entry, 256);
+	if (g_strcmp0(title_entry, NULL)!=0) gtk_entry_set_text(GTK_ENTRY(title_entry), g_strconcat(g_get_user_special_dir(G_USER_DIRECTORY_MUSIC), "/", g_get_real_name(), ".voice", NULL)); else gtk_entry_set_text(GTK_ENTRY(title_entry), gtk_entry_get_text(GTK_ENTRY(title_entry)));
 	gtk_box_pack_start(GTK_BOX(page[1].widget), GTK_WIDGET(title_label),
 			   FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(page[1].widget), GTK_WIDGET(title_entry),
@@ -708,7 +742,7 @@ main (gint argc, gchar **argv)
 	page[2].widget = gtk_box_new(FALSE, 5);
 	filename_label = gtk_label_new(_("Filename:"));
 	filename_entry = gtk_entry_new();
-	if (g_strcmp0(filename_entry, NULL)!=0) gtk_entry_set_text(GTK_ENTRY(filename_entry), g_strconcat ("GNOME.ogg", NULL)); else gtk_entry_set_text(GTK_ENTRY(filename_entry), gtk_entry_get_text(GTK_ENTRY(filename_entry)));
+	if (g_strcmp0(filename_entry, NULL)!=0) gtk_entry_set_text(GTK_ENTRY(filename_entry), g_strconcat (g_get_user_special_dir(G_USER_DIRECTORY_MUSIC), "/", "GNOME.ogg", NULL)); else gtk_entry_set_text(GTK_ENTRY(filename_entry), gtk_entry_get_text(GTK_ENTRY(filename_entry)));
 	gtk_box_pack_start(GTK_BOX(page[2].widget), GTK_WIDGET(filename_label),
 			   FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(page[2].widget), GTK_WIDGET(filename_entry),
@@ -717,9 +751,8 @@ main (gint argc, gchar **argv)
 			 G_CALLBACK(gv_wizard_apply),
 			 gtk_entry_get_text(GTK_ENTRY(filename_entry)));
 	page[3].widget = gtk_box_new(FALSE, 5);
-	summary_label = gtk_label_new(_("Recording"));
-	summary_entry = gtk_entry_new();
-	gtk_entry_set_text(GTK_ENTRY(summary_entry), "Recording");
+	summary_label = gtk_label_new(_("Summary:"));
+        summary_entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(page[4].widget), GTK_WIDGET(summary_label),
 			   FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(page[4].widget), GTK_WIDGET(summary_entry),
@@ -774,7 +807,7 @@ main (gint argc, gchar **argv)
 				 GST_TAG_TITLE, g_get_real_name(),
 				 GST_TAG_ARTIST, g_get_real_name(),
 				 GST_TAG_ALBUM, gtk_entry_get_text(filename_entry),
-				 GST_TAG_COMMENT, "Voice 1.0.1",
+				 GST_TAG_COMMENT, "Voice 1.1.0",
 				 GST_TAG_DATE, g_date_time_format_iso8601 (datestamp),
 				 NULL);
 	g_date_time_unref (datestamp);
@@ -820,7 +853,7 @@ main (gint argc, gchar **argv)
 	/* oscilloscope_visual = create_oscilloscope_visual (); */
         /* gnome_voice_add_visual_oscilloscope (layer, GNOME_VOICE_MARKER (oscilloscope_visual)); */
 
-#if 0
+
 	gclue_simple_new ("gnome-voice",
 			  accuracy_level,
 			  time_threshold,
@@ -828,17 +861,17 @@ main (gint argc, gchar **argv)
 			  CHAMPLAIN_VIEW (view));
 
 	location = gclue_simple_get_location (GCLUE_SIMPLE(simple));
-#endif
+
 	/* Finish initialising the map view */
 	g_object_set (G_OBJECT (actor), "zoom-level", 1,
 		      "kinetic-mode", TRUE, NULL);
+	champlain_view_center_on (CHAMPLAIN_VIEW (actor), atof(voiceinfo->location->lat), atof(voiceinfo->location->lon));
 #if 0
-	champlain_view_center_on (CHAMPLAIN_VIEW (actor), (gdouble)*voiceinfo->location->lat, (gdouble)*voiceinfo->location->lon);
-#endif
 	champlain_view_center_on (CHAMPLAIN_VIEW (actor), lat_gps, lon_gps);
+#endif
 	g_object_set (G_OBJECT (second), "zoom-level", 6,
 		      "kinetic-mode", TRUE, NULL);
-	champlain_view_center_on (CHAMPLAIN_VIEW (second), lat, lon);
+	champlain_view_center_on (CHAMPLAIN_VIEW (second), lat_gps, lon_gps);
 	/* Create callback that updates the map periodically */
 	callback_data.view = CHAMPLAIN_VIEW (actor);
 	callback_data.voice_marker = CHAMPLAIN_MARKER (voice_marker);
