@@ -3,11 +3,10 @@
 
 #include <geoclue.h>
 
-double lat = 34.0223563;
-double lon = -118.2873057;
-
-double lat_gps = 34.0223563;
-double lon_gps = -118.2873057;
+double lat = 68.1841843;
+double lon = 15.5974169;
+double lat_gps = 68.1841843;
+double lon_gps = 15.5974169;
 
 typedef struct
 {
